/**
 * This class provides various routines to
 * convert metric measurements to U.S. units and
 * vice versa.
 */

class MetricConverter {

//----------------------------------
//    Data Members
//----------------------------------
  
  /**
   * A factor to convert inches to centimeters
   */
  public static final double INCHES_TO_CENTIMETERS = 2.54; 
   
  /**
   * A factor to convert centimeters to inches
   */
  public static final double CENTIMETERS_TO_INCHES = 1 / INCHES_TO_CENTIMETERS;
   
  /**
   * A factor to convert feet to inches
   */
  public static final double FEET_TO_INCHES        = 12.0;


//----------------------------------
//    Constructors
//----------------------------------
                                
   /**
    * Default constructor (THAT YOU SHOULD NOT USE)
    */
   public MetricConverter( ) {
       System.out.println("You DON'T need to construct an instance of " +
                          "this class to use class methods!");
       System.exit(1);
       
   }   
   
//-------------------------------------------------
//  Public Class Methods:
// 
//      static double  inchesToCentimeters        ( double         )
//      static double  centimetersToInches        ( double         )
//      static double  feetAndInchesToCentimeters ( double, double )
//------------------------------------------------
      
   /**
    * Converts a given length in inches to
    * equivalent centimeters.
    * 
    * @param inches  the length expressed in inches
    * @return length expressed in centimeters
    */
   public static double inchesToCentimeters( double inches ) {
      return inches * INCHES_TO_CENTIMETERS;
   }
   

   /**
    * Converts a given length in feet and inches to
    * equivalent centimeters.
    * 
    * @param centimeters the length expressed in centimeters
    * @return length expressed in inches
    */
   public static double centimetersToInches( double centimeters ) {
       return centimeters *  CENTIMETERS_TO_INCHES;
   }

   /**
    * Converts a given length in feet and inches to
    * equivalent centimeters.
    * 
    * @param feet   the feet portion of the length
    * @param inches the inch portion of the length
    * @return length expressed in centimeters
    */
   public static double feetAndInchesToCentimeters(double feet, double inches) {
      return inchesToCentimeters(feet * FEET_TO_INCHES + inches);
   }                
}
